@@ -311,6 +311,11 @@ func (service *serviceType) handleIncomingPacket(bytes []byte) (result outgoingP
 				bytes: service.handleFormatMessagesRequest(p.id, request),
 			}
 
+		case "analyze-metafile":
+			return outgoingPacket{
+				bytes: service.handleAnalyzeMetafileRequest(p.id, request),
+			}
+
 		default:
 			return outgoingPacket{
 				bytes: encodePacket(packet{
@@ -906,6 +911,27 @@ func (service *serviceType) handleFormatMessagesRequest(id uint32, request map[s
 	})
 }
 
+func (service *serviceType) handleAnalyzeMetafileRequest(id uint32, request map[string]interface{}) []byte {
+	metafile := request["metafile"].(string)
+
+	options := api.AnalyzeMetafileOptions{}
+	if value, ok := request["color"].(bool); ok {
+		options.Color = value
+	}
+	if value, ok := request["verbose"].(bool); ok {
+		options.Verbose = value
+	}
+
+	result := api.AnalyzeMetafile(metafile, options)
+
+	return encodePacket(packet{
+		id: id,
+		value: map[string]interface{}{
+			"result": result,
+		},
+	})
+}
+
 func encodeStringArray(strings []string) []interface{} {
 	values := make([]interface{}, len(strings))
 	for i, value := range strings {
