@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"math/rand"
 	"net/http"
+	"runtime/debug"
 	"sort"
 	"strings"
 	"sync"
@@ -30,6 +31,7 @@ import (
 	"github.com/evanw/esbuild/internal/logger"
 	"github.com/evanw/esbuild/internal/resolver"
 	"github.com/evanw/esbuild/internal/runtime"
+	"github.com/evanw/esbuild/internal/sourcemap"
 	"github.com/evanw/esbuild/internal/xxhash"
 )
 
@@ -50,7 +52,7 @@ type dataForSourceMap struct {
 	// This data is for the printer. It maps from byte offsets in the file (which
 	// are stored at every AST node) to UTF-16 column offsets (required by source
 	// maps).
-	lineOffsetTables []js_printer.LineOffsetTable
+	lineOffsetTables []sourcemap.LineOffsetTable
 
 	// This contains the quoted contents of the original source file. It's what
 	// needs to be embedded in the "sourcesContent" array in the final source
@@ -170,6 +172,21 @@ func parseFile(args parseArgs) {
 		},
 	}
 
+	defer func() {
+		r := recover()
+		if r != nil {
+			stack := strings.TrimSpace(string(debug.Stack()))
+			tracker := logger.MakeLineColumnTracker(&source)
+			data := logger.RangeData(&tracker, logger.Range{}, fmt.Sprintf("panic: %v", r))
+			data.Location.LineText = fmt.Sprintf("%s\n%s", data.Location.LineText, stack)
+			args.log.AddMsg(logger.Msg{
+				Kind: logger.Error,
+				Data: data,
+			})
+			args.results <- result
+		}
+	}()
+
 	switch loader {
 	case config.LoaderJS:
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
@@ -182,8 +199,9 @@ func parseFile(args parseArgs) {
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
 
-	case config.LoaderTS:
+	case config.LoaderTS, config.LoaderTSNoAmbiguousLessThan:
 		args.options.TS.Parse = true
+		args.options.TS.NoAmbiguousLessThan = loader == config.LoaderTSNoAmbiguousLessThan
 		ast, ok := args.caches.JSCache.Parse(args.log, source, js_parser.OptionsFromConfig(&args.options))
 		result.file.inputFile.Repr = &graph.JSRepr{AST: ast}
 		result.ok = ok
@@ -446,9 +464,16 @@ func parseFile(args parseArgs) {
 
 	// Attempt to parse the source map if present
 	if loader.CanHaveSourceMap() && args.options.SourceMap != config.SourceMapNone {
-		if repr, ok := result.file.inputFile.Repr.(*graph.JSRepr); ok && repr.AST.SourceMapComment.Text != "" {
+		var sourceMapComment logger.Span
+		switch repr := result.file.inputFile.Repr.(type) {
+		case *graph.JSRepr:
+			sourceMapComment = repr.AST.SourceMapComment
+		case *graph.CSSRepr:
+			sourceMapComment = repr.AST.SourceMapComment
+		}
+		if sourceMapComment.Text != "" {
 			if path, contents := extractSourceMapFromComment(args.log, args.fs, &args.caches.FSCache,
-				args.res, &source, repr.AST.SourceMapComment, absResolveDir); contents != nil {
+				args.res, &source, sourceMapComment, absResolveDir); contents != nil {
 				result.file.inputFile.InputSourceMap = js_parser.ParseSourceMap(args.log, logger.Source{
 					KeyPath:    path,
 					PrettyPath: args.res.PrettyPath(path),
@@ -515,7 +540,7 @@ func extractSourceMapFromComment(
 	fsCache *cache.FSCache,
 	res resolver.Resolver,
 	source *logger.Source,
-	comment js_ast.Span,
+	comment logger.Span,
 	absResolveDir string,
 ) (logger.Path, *string) {
 	tracker := logger.MakeLineColumnTracker(source)
@@ -673,7 +698,9 @@ func runOnResolvePlugins(
 				fsCache.ReadFile(fs, file)
 			}
 			for _, dir := range result.AbsWatchDirs {
-				fs.ReadDirectory(dir)
+				if entries, err, _ := fs.ReadDirectory(dir); err == nil {
+					entries.SortedKeys()
+				}
 			}
 
 			// Stop now if there was an error
@@ -788,7 +815,9 @@ func runOnLoadPlugins(
 				fsCache.ReadFile(fs, file)
 			}
 			for _, dir := range result.AbsWatchDirs {
-				fs.ReadDirectory(dir)
+				if entries, err, _ := fs.ReadDirectory(dir); err == nil {
+					entries.SortedKeys()
+				}
 			}
 
 			// Stop now if there was an error
@@ -1076,9 +1105,9 @@ func (s *scanner) maybeParseFile(
 	optionsClone.TSTarget = resolveResult.TSTarget
 
 	// Set the module type preference using node's module type rules
-	if strings.HasSuffix(path.Text, ".mjs") {
+	if strings.HasSuffix(path.Text, ".mjs") || strings.HasSuffix(path.Text, ".mts") {
 		optionsClone.ModuleType = config.ModuleESM
-	} else if strings.HasSuffix(path.Text, ".cjs") {
+	} else if strings.HasSuffix(path.Text, ".cjs") || strings.HasSuffix(path.Text, ".cts") {
 		optionsClone.ModuleType = config.ModuleCommonJS
 	} else {
 		optionsClone.ModuleType = resolveResult.ModuleType
@@ -1899,6 +1928,8 @@ func DefaultExtensionToLoaderMap() map[string]config.Loader {
 		".cjs":  config.LoaderJS,
 		".jsx":  config.LoaderJSX,
 		".ts":   config.LoaderTS,
+		".cts":  config.LoaderTSNoAmbiguousLessThan,
+		".mts":  config.LoaderTSNoAmbiguousLessThan,
 		".tsx":  config.LoaderTSX,
 		".css":  config.LoaderCSS,
 		".json": config.LoaderJSON,
@@ -2131,41 +2162,46 @@ func (b *Bundle) computeDataForSourceMapsInParallel(options *config.Options, rea
 
 	for _, sourceIndex := range reachableFiles {
 		if f := &b.files[sourceIndex]; f.inputFile.Loader.CanHaveSourceMap() {
-			if repr, ok := f.inputFile.Repr.(*graph.JSRepr); ok {
-				waitGroup.Add(1)
-				go func(sourceIndex uint32, f *scannerFile, repr *graph.JSRepr) {
-					result := &results[sourceIndex]
-					result.lineOffsetTables = js_printer.GenerateLineOffsetTables(f.inputFile.Source.Contents, repr.AST.ApproximateLineCount)
-					sm := f.inputFile.InputSourceMap
-					if !options.ExcludeSourcesContent {
-						if sm == nil {
-							// Simple case: no nested source map
-							result.quotedContents = [][]byte{js_printer.QuoteForJSON(f.inputFile.Source.Contents, options.ASCIIOnly)}
-						} else {
-							// Complex case: nested source map
-							result.quotedContents = make([][]byte, len(sm.Sources))
-							nullContents := []byte("null")
-							for i := range sm.Sources {
-								// Missing contents become a "null" literal
-								quotedContents := nullContents
-								if i < len(sm.SourcesContent) {
-									if value := sm.SourcesContent[i]; value.Quoted != "" {
-										if options.ASCIIOnly && !isASCIIOnly(value.Quoted) {
-											// Re-quote non-ASCII values if output is ASCII-only
-											quotedContents = js_printer.QuoteForJSON(js_lexer.UTF16ToString(value.Value), options.ASCIIOnly)
-										} else {
-											// Otherwise just use the value directly from the input file
-											quotedContents = []byte(value.Quoted)
-										}
+			var approximateLineCount int32
+			switch repr := f.inputFile.Repr.(type) {
+			case *graph.JSRepr:
+				approximateLineCount = repr.AST.ApproximateLineCount
+			case *graph.CSSRepr:
+				approximateLineCount = repr.AST.ApproximateLineCount
+			}
+			waitGroup.Add(1)
+			go func(sourceIndex uint32, f *scannerFile, approximateLineCount int32) {
+				result := &results[sourceIndex]
+				result.lineOffsetTables = sourcemap.GenerateLineOffsetTables(f.inputFile.Source.Contents, approximateLineCount)
+				sm := f.inputFile.InputSourceMap
+				if !options.ExcludeSourcesContent {
+					if sm == nil {
+						// Simple case: no nested source map
+						result.quotedContents = [][]byte{js_printer.QuoteForJSON(f.inputFile.Source.Contents, options.ASCIIOnly)}
+					} else {
+						// Complex case: nested source map
+						result.quotedContents = make([][]byte, len(sm.Sources))
+						nullContents := []byte("null")
+						for i := range sm.Sources {
+							// Missing contents become a "null" literal
+							quotedContents := nullContents
+							if i < len(sm.SourcesContent) {
+								if value := sm.SourcesContent[i]; value.Quoted != "" {
+									if options.ASCIIOnly && !isASCIIOnly(value.Quoted) {
+										// Re-quote non-ASCII values if output is ASCII-only
+										quotedContents = js_printer.QuoteForJSON(js_lexer.UTF16ToString(value.Value), options.ASCIIOnly)
+									} else {
+										// Otherwise just use the value directly from the input file
+										quotedContents = []byte(value.Quoted)
 									}
 								}
-								result.quotedContents[i] = quotedContents
 							}
+							result.quotedContents[i] = quotedContents
 						}
 					}
-					waitGroup.Done()
-				}(sourceIndex, f, repr)
-			}
+				}
+				waitGroup.Done()
+			}(sourceIndex, f, approximateLineCount)
 		}
 	}
 
@@ -2281,7 +2317,7 @@ func (cache *runtimeCache) parseRuntime(options *config.Options) (source logger.
 
 		// Always do tree shaking for the runtime because we never want to
 		// include unnecessary runtime code
-		Mode: config.ModeBundle,
+		TreeShaking: true,
 	}))
 	if log.HasErrors() {
 		msgs := "Internal error: failed to parse runtime:\n"
