@@ -16,7 +16,7 @@ func (p *parser) commaToken() css_ast.Token {
 	return t
 }
 
-func expandTokenQuad(tokens []css_ast.Token) (result [4]css_ast.Token, ok bool) {
+func expandTokenQuad(tokens []css_ast.Token, allowedIdent string) (result [4]css_ast.Token, ok bool) {
 	n := len(tokens)
 	if n < 1 || n > 4 {
 		return
@@ -24,7 +24,7 @@ func expandTokenQuad(tokens []css_ast.Token) (result [4]css_ast.Token, ok bool)
 
 	// Don't do this if we encounter any unexpected tokens such as "var()"
 	for i := 0; i < n; i++ {
-		if !tokens[i].Kind.IsNumericOrIdent() {
+		if t := tokens[i]; !t.Kind.IsNumeric() && (t.Kind != css_lexer.TIdent || allowedIdent == "" || t.Text != allowedIdent) {
 			return
 		}
 	}
@@ -76,13 +76,13 @@ func compactTokenQuad(a css_ast.Token, b css_ast.Token, c css_ast.Token, d css_a
 	return tokens
 }
 
-func (p *parser) processDeclarations(rules []css_ast.R) []css_ast.R {
+func (p *parser) processDeclarations(rules []css_ast.Rule) []css_ast.Rule {
 	margin := boxTracker{}
 	padding := boxTracker{}
 	borderRadius := borderRadiusTracker{}
 
 	for i, rule := range rules {
-		decl, ok := rule.(*css_ast.RDeclaration)
+		decl, ok := rule.Data.(*css_ast.RDeclaration)
 		if !ok {
 			continue
 		}
@@ -114,7 +114,10 @@ func (p *parser) processDeclarations(rules []css_ast.R) []css_ast.R {
 				decl.Value[0] = p.lowerColor(decl.Value[0])
 
 				if p.options.MangleSyntax {
-					decl.Value[0] = p.mangleColor(decl.Value[0])
+					t := decl.Value[0]
+					if hex, ok := parseColor(t); ok {
+						decl.Value[0] = p.mangleColor(t, hex)
+					}
 				}
 			}
 
@@ -197,7 +200,7 @@ func (p *parser) processDeclarations(rules []css_ast.R) []css_ast.R {
 	if p.options.MangleSyntax {
 		end := 0
 		for _, rule := range rules {
-			if rule != nil {
+			if rule.Data != nil {
 				rules[end] = rule
 				end++
 			}
