@@ -15,16 +15,18 @@ import (
 // support for parsing https://drafts.csswg.org/css-nesting-1/.
 
 type parser struct {
-	log           logger.Log
-	source        logger.Source
-	tracker       logger.LineColumnTracker
-	options       Options
-	tokens        []css_lexer.Token
-	stack         []css_lexer.T
-	index         int
-	end           int
-	prevError     logger.Loc
-	importRecords []ast.ImportRecord
+	log                 logger.Log
+	source              logger.Source
+	tracker             logger.LineColumnTracker
+	options             Options
+	tokens              []css_lexer.Token
+	licenseComments     []css_lexer.Comment
+	stack               []css_lexer.T
+	index               int
+	end                 int
+	licenseCommentIndex int
+	prevError           logger.Loc
+	importRecords       []ast.ImportRecord
 }
 
 type Options struct {
@@ -34,23 +36,28 @@ type Options struct {
 }
 
 func Parse(log logger.Log, source logger.Source, options Options) css_ast.AST {
+	result := css_lexer.Tokenize(log, source)
 	p := parser{
-		log:       log,
-		source:    source,
-		tracker:   logger.MakeLineColumnTracker(&source),
-		options:   options,
-		tokens:    css_lexer.Tokenize(log, source),
-		prevError: logger.Loc{Start: -1},
+		log:             log,
+		source:          source,
+		tracker:         logger.MakeLineColumnTracker(&source),
+		options:         options,
+		tokens:          result.Tokens,
+		licenseComments: result.LicenseComments,
+		prevError:       logger.Loc{Start: -1},
 	}
 	p.end = len(p.tokens)
-	tree := css_ast.AST{}
-	tree.Rules = p.parseListOfRules(ruleContext{
+	rules := p.parseListOfRules(ruleContext{
 		isTopLevel:     true,
 		parseSelectors: true,
 	})
-	tree.ImportRecords = p.importRecords
 	p.expect(css_lexer.TEndOfFile)
-	return tree
+	return css_ast.AST{
+		Rules:                rules,
+		ImportRecords:        p.importRecords,
+		ApproximateLineCount: result.ApproximateLineCount,
+		SourceMapComment:     result.SourceMapComment,
+	}
 }
 
 func (p *parser) advance() {
@@ -154,14 +161,28 @@ type ruleContext struct {
 	parseSelectors bool
 }
 
-func (p *parser) parseListOfRules(context ruleContext) []css_ast.R {
+func (p *parser) parseListOfRules(context ruleContext) []css_ast.Rule {
 	didWarnAboutCharset := false
 	didWarnAboutImport := false
-	rules := []css_ast.R{}
+	rules := []css_ast.Rule{}
 	locs := []logger.Loc{}
 
 loop:
 	for {
+		// If there are any license comments immediately before the current token,
+		// turn them all into comment rules and append them to the current rule list
+		for p.licenseCommentIndex < len(p.licenseComments) {
+			comment := p.licenseComments[p.licenseCommentIndex]
+			if comment.TokenIndexAfter != uint32(p.index) {
+				break
+			}
+			rules = append(rules, css_ast.Rule{Loc: comment.Loc, Data: &css_ast.RComment{Text: comment.Text}})
+			if context.isTopLevel {
+				locs = append(locs, comment.Loc)
+			}
+			p.licenseCommentIndex++
+		}
+
 		switch p.current().Kind {
 		case css_lexer.TEndOfFile, css_lexer.TCloseBrace:
 			break loop
@@ -176,21 +197,25 @@ loop:
 
 			// Validate structure
 			if context.isTopLevel {
-				switch rule.(type) {
+				switch rule.Data.(type) {
 				case *css_ast.RAtCharset:
-					if !didWarnAboutCharset && len(rules) > 0 {
-						p.log.AddRangeWarningWithNotes(&p.tracker, first, "\"@charset\" must be the first rule in the file",
-							[]logger.MsgData{logger.RangeData(&p.tracker, logger.Range{Loc: locs[len(locs)-1]},
-								"This rule cannot come before a \"@charset\" rule")})
-						didWarnAboutCharset = true
+					if !didWarnAboutCharset {
+						for i, before := range rules {
+							if _, ok := before.Data.(*css_ast.RComment); !ok {
+								p.log.AddRangeWarningWithNotes(&p.tracker, first, "\"@charset\" must be the first rule in the file",
+									[]logger.MsgData{logger.RangeData(&p.tracker, logger.Range{Loc: locs[i]},
+										"This rule cannot come before a \"@charset\" rule")})
+								didWarnAboutCharset = true
+							}
+						}
 					}
 
 				case *css_ast.RAtImport:
 					if !didWarnAboutImport {
 					importLoop:
 						for i, before := range rules {
-							switch before.(type) {
-							case *css_ast.RAtCharset, *css_ast.RAtImport:
+							switch before.Data.(type) {
+							case *css_ast.RComment, *css_ast.RAtCharset, *css_ast.RAtImport:
 							default:
 								p.log.AddRangeWarningWithNotes(&p.tracker, first, "All \"@import\" rules must come first",
 									[]logger.MsgData{logger.RangeData(&p.tracker, logger.Range{Loc: locs[i]},
@@ -232,7 +257,7 @@ loop:
 	return rules
 }
 
-func (p *parser) parseListOfDeclarations() (list []css_ast.R) {
+func (p *parser) parseListOfDeclarations() (list []css_ast.Rule) {
 	for {
 		switch p.current().Kind {
 		case css_lexer.TWhitespace, css_lexer.TSemicolon:
@@ -260,7 +285,7 @@ func (p *parser) parseListOfDeclarations() (list []css_ast.R) {
 	}
 }
 
-func removeEmptyAndDuplicateRules(rules []css_ast.R) []css_ast.R {
+func removeEmptyAndDuplicateRules(rules []css_ast.Rule) []css_ast.Rule {
 	type hashEntry struct {
 		indices []uint32
 	}
@@ -274,11 +299,11 @@ skipRule:
 	for i := n - 1; i >= 0; i-- {
 		rule := rules[i]
 
-		switch r := rule.(type) {
+		switch r := rule.Data.(type) {
 		case *css_ast.RAtKeyframes:
-			if len(r.Blocks) == 0 {
-				continue
-			}
+			// Do not remove empty "@keyframe foo {}" rules. Even empty rules still
+			// dispatch JavaScript animation events, so removing them changes
+			// behavior: https://bugzilla.mozilla.org/show_bug.cgi?id=1004377.
 
 		case *css_ast.RKnownAt:
 			if len(r.Rules) == 0 {
@@ -291,12 +316,12 @@ skipRule:
 			}
 		}
 
-		if hash, ok := rule.Hash(); ok {
+		if hash, ok := rule.Data.Hash(); ok {
 			entry := entries[hash]
 
 			// For duplicate rules, omit all but the last copy
 			for _, index := range entry.indices {
-				if rule.Equal(rules[index]) {
+				if rule.Data.Equal(rules[index].Data) {
 					continue skipRule
 				}
 			}
@@ -408,7 +433,7 @@ type atRuleContext struct {
 	isDeclarationList bool
 }
 
-func (p *parser) parseAtRule(context atRuleContext) css_ast.R {
+func (p *parser) parseAtRule(context atRuleContext) css_ast.Rule {
 	// Parse the name
 	atToken := p.decoded()
 	atRange := p.current().Range
@@ -423,13 +448,13 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.R {
 		p.expect(css_lexer.TWhitespace)
 		if p.peek(css_lexer.TString) {
 			encoding := p.decoded()
-			if encoding != "UTF-8" {
+			if !strings.EqualFold(encoding, "UTF-8") {
 				p.log.AddRangeWarning(&p.tracker, p.current().Range,
 					fmt.Sprintf("\"UTF-8\" will be used instead of unsupported charset %q", encoding))
 			}
 			p.advance()
 			p.expect(css_lexer.TSemicolon)
-			return &css_ast.RAtCharset{Encoding: encoding}
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtCharset{Encoding: encoding}}
 		}
 		p.expect(css_lexer.TString)
 
@@ -461,10 +486,10 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.R {
 				Path:  logger.Path{Text: path},
 				Range: r,
 			})
-			return &css_ast.RAtImport{
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtImport{
 				ImportRecordIndex: importRecordIndex,
 				ImportConditions:  importConditions,
-			}
+			}}
 		}
 
 	case "keyframes", "-webkit-keyframes", "-moz-keyframes", "-ms-keyframes", "-o-keyframes":
@@ -556,11 +581,11 @@ func (p *parser) parseAtRule(context atRuleContext) css_ast.R {
 			}
 
 			p.expect(css_lexer.TCloseBrace)
-			return &css_ast.RAtKeyframes{
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RAtKeyframes{
 				AtToken: atToken,
 				Name:    name,
 				Blocks:  blocks,
-			}
+			}}
 		}
 
 	default:
@@ -599,12 +624,12 @@ prelude:
 			if kind != atRuleEmpty && kind != atRuleUnknown {
 				p.expect(css_lexer.TOpenBrace)
 				p.eat(css_lexer.TSemicolon)
-				return &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}
+				return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}}
 			}
 
 			// Otherwise, parse an unknown at rule
 			p.expect(css_lexer.TSemicolon)
-			return &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}
+			return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude}}
 
 		default:
 			p.parseComponentValue()
@@ -619,19 +644,19 @@ prelude:
 		p.expect(css_lexer.TSemicolon)
 		p.parseBlock(css_lexer.TOpenBrace, css_lexer.TCloseBrace)
 		block := p.convertTokens(p.tokens[blockStart:p.index])
-		return &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}
+		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
 
 	case atRuleDeclarations:
 		// Parse known rules whose blocks consist of whatever the current context is
 		p.advance()
 		rules := p.parseListOfDeclarations()
 		p.expect(css_lexer.TCloseBrace)
-		return &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}
+		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
 	case atRuleInheritContext:
 		// Parse known rules whose blocks consist of whatever the current context is
 		p.advance()
-		var rules []css_ast.R
+		var rules []css_ast.Rule
 		if context.isDeclarationList {
 			rules = p.parseListOfDeclarations()
 		} else {
@@ -640,13 +665,13 @@ prelude:
 			})
 		}
 		p.expect(css_lexer.TCloseBrace)
-		return &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}
+		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RKnownAt{AtToken: atToken, Prelude: prelude, Rules: rules}}
 
 	default:
 		// Otherwise, parse an unknown rule
 		p.parseBlock(css_lexer.TOpenBrace, css_lexer.TCloseBrace)
 		block, _ := p.convertTokensHelper(p.tokens[blockStart:p.index], css_lexer.TEndOfFile, convertTokensOpts{allowImports: true})
-		return &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}
+		return css_ast.Rule{Loc: atRange.Loc, Data: &css_ast.RUnknownAt{AtToken: atToken, Prelude: prelude, Block: block}}
 	}
 }
 
@@ -656,8 +681,9 @@ func (p *parser) convertTokens(tokens []css_lexer.Token) []css_ast.Token {
 }
 
 type convertTokensOpts struct {
-	allowImports       bool
-	verbatimWhitespace bool
+	allowImports         bool
+	verbatimWhitespace   bool
+	isInsideCalcFunction bool
 }
 
 func (p *parser) convertTokensHelper(tokens []css_lexer.Token, close css_lexer.T, opts convertTokensOpts) ([]css_ast.Token, []css_lexer.Token) {
@@ -678,6 +704,14 @@ loop:
 		}
 		nextWhitespace = 0
 
+		// Warn about invalid "+" and "-" operators that break the containing "calc()"
+		if opts.isInsideCalcFunction && t.Kind.IsNumeric() && len(result) > 0 && result[len(result)-1].Kind.IsNumeric() &&
+			(strings.HasPrefix(token.Text, "+") || strings.HasPrefix(token.Text, "-")) {
+			// "calc(1+2)" and "calc(1-2)" are invalid
+			p.log.AddRangeWarning(&p.tracker, logger.Range{Loc: t.Range.Loc, Len: 1},
+				fmt.Sprintf("The %q operator only works if there is whitespace on both sides", token.Text[:1]))
+		}
+
 		switch t.Kind {
 		case css_lexer.TWhitespace:
 			if last := len(result) - 1; last >= 0 {
@@ -686,6 +720,20 @@ loop:
 			nextWhitespace = css_ast.WhitespaceBefore
 			continue
 
+		case css_lexer.TDelimPlus, css_lexer.TDelimMinus:
+			// Warn about invalid "+" and "-" operators that break the containing "calc()"
+			if opts.isInsideCalcFunction && len(tokens) > 0 {
+				if len(result) == 0 || result[len(result)-1].Kind == css_lexer.TComma {
+					// "calc(-(1 + 2))" is invalid
+					p.log.AddRangeWarning(&p.tracker, t.Range,
+						fmt.Sprintf("%q can only be used as an infix operator, not a prefix operator", token.Text))
+				} else if token.Whitespace != css_ast.WhitespaceBefore || tokens[0].Kind != css_lexer.TWhitespace {
+					// "calc(1- 2)" and "calc(1 -(2))" are invalid
+					p.log.AddRangeWarning(&p.tracker, t.Range,
+						fmt.Sprintf("The %q operator only works if there is whitespace on both sides", token.Text))
+				}
+			}
+
 		case css_lexer.TNumber:
 			if p.options.MangleSyntax {
 				if text, ok := mangleNumber(token.Text); ok {
@@ -733,9 +781,17 @@ loop:
 				// CSS variables require verbatim whitespace for correctness
 				nestedOpts.verbatimWhitespace = true
 			}
+			if token.Text == "calc" {
+				nestedOpts.isInsideCalcFunction = true
+			}
 			nested, tokens = p.convertTokensHelper(tokens, css_lexer.TCloseParen, nestedOpts)
 			token.Children = &nested
 
+			// Apply "calc" simplification rules when minifying
+			if p.options.MangleSyntax && token.Text == "calc" {
+				token = p.tryToReduceCalcExpression(token)
+			}
+
 			// Treat a URL function call with a string just like a URL token
 			if token.Text == "url" && len(nested) == 1 && nested[0].Kind == css_lexer.TString {
 				token.Kind = css_lexer.TURL
@@ -926,16 +982,16 @@ func mangleNumber(t string) (string, bool) {
 	return t, t != original
 }
 
-func (p *parser) parseSelectorRule() css_ast.R {
+func (p *parser) parseSelectorRule() css_ast.Rule {
 	preludeStart := p.index
 
 	// Try parsing the prelude as a selector list
 	if list, ok := p.parseSelectorList(); ok {
-		rule := css_ast.RSelector{Selectors: list}
+		selector := css_ast.RSelector{Selectors: list}
 		if p.expect(css_lexer.TOpenBrace) {
-			rule.Rules = p.parseListOfDeclarations()
+			selector.Rules = p.parseListOfDeclarations()
 			p.expect(css_lexer.TCloseBrace)
-			return &rule
+			return css_ast.Rule{Loc: p.tokens[preludeStart].Range.Loc, Data: &selector}
 		}
 	}
 
@@ -943,7 +999,9 @@ func (p *parser) parseSelectorRule() css_ast.R {
 	return p.parseQualifiedRuleFrom(preludeStart, true /* isAlreadyInvalid */)
 }
 
-func (p *parser) parseQualifiedRuleFrom(preludeStart int, isAlreadyInvalid bool) *css_ast.RQualified {
+func (p *parser) parseQualifiedRuleFrom(preludeStart int, isAlreadyInvalid bool) css_ast.Rule {
+	preludeLoc := p.tokens[preludeStart].Range.Loc
+
 loop:
 	for {
 		switch p.current().Kind {
@@ -957,30 +1015,31 @@ loop:
 			}
 			prelude := p.convertTokens(p.tokens[preludeStart:p.index])
 			p.advance()
-			return &css_ast.RQualified{Prelude: prelude}
+			return css_ast.Rule{Loc: preludeLoc, Data: &css_ast.RQualified{Prelude: prelude}}
 
 		default:
 			p.parseComponentValue()
 		}
 	}
 
-	rule := css_ast.RQualified{
+	qualified := css_ast.RQualified{
 		Prelude: p.convertTokens(p.tokens[preludeStart:p.index]),
 	}
 
 	if p.eat(css_lexer.TOpenBrace) {
-		rule.Rules = p.parseListOfDeclarations()
+		qualified.Rules = p.parseListOfDeclarations()
 		p.expect(css_lexer.TCloseBrace)
 	} else if !isAlreadyInvalid {
 		p.expect(css_lexer.TOpenBrace)
 	}
 
-	return &rule
+	return css_ast.Rule{Loc: preludeLoc, Data: &qualified}
 }
 
-func (p *parser) parseDeclaration() css_ast.R {
+func (p *parser) parseDeclaration() css_ast.Rule {
 	// Parse the key
 	keyStart := p.index
+	keyLoc := p.tokens[keyStart].Range.Loc
 	ok := false
 	if p.expect(css_lexer.TIdent) {
 		p.eat(css_lexer.TWhitespace)
@@ -1013,9 +1072,9 @@ stop:
 
 	// Stop now if this is not a valid declaration
 	if !ok {
-		return &css_ast.RBadDeclaration{
+		return css_ast.Rule{Loc: keyLoc, Data: &css_ast.RBadDeclaration{
 			Tokens: p.convertTokens(p.tokens[keyStart:p.index]),
-		}
+		}}
 	}
 
 	keyToken := p.tokens[keyStart]
@@ -1056,13 +1115,13 @@ stop:
 		}
 	}
 
-	return &css_ast.RDeclaration{
+	return css_ast.Rule{Loc: keyLoc, Data: &css_ast.RDeclaration{
 		Key:       css_ast.KnownDeclarations[keyText],
 		KeyText:   keyText,
 		KeyRange:  keyToken.Range,
 		Value:     result,
 		Important: important,
-	}
+	}}
 }
 
 func (p *parser) parseComponentValue() {
