@@ -260,11 +260,20 @@ const (
 	PropertyGet
 	PropertySet
 	PropertySpread
+	PropertyDeclare
+	PropertyClassStaticBlock
 )
 
+type ClassStaticBlock struct {
+	Loc   logger.Loc
+	Stmts []Stmt
+}
+
 type Property struct {
-	TSDecorators []Expr
-	Key          Expr
+	TSDecorators     []Expr
+	ClassStaticBlock *ClassStaticBlock
+
+	Key Expr
 
 	// This is omitted for class fields
 	ValueOrNil Expr
@@ -350,6 +359,11 @@ type Binding struct {
 // Go's type system.
 type B interface{ isBinding() }
 
+func (*BMissing) isBinding()    {}
+func (*BIdentifier) isBinding() {}
+func (*BArray) isBinding()      {}
+func (*BObject) isBinding()     {}
+
 type BMissing struct{}
 
 type BIdentifier struct{ Ref Ref }
@@ -365,11 +379,6 @@ type BObject struct {
 	IsSingleLine bool
 }
 
-func (*BMissing) isBinding()    {}
-func (*BIdentifier) isBinding() {}
-func (*BArray) isBinding()      {}
-func (*BObject) isBinding()     {}
-
 type Expr struct {
 	Loc  logger.Loc
 	Data E
@@ -379,6 +388,43 @@ type Expr struct {
 // Go's type system.
 type E interface{ isExpr() }
 
+func (*EArray) isExpr()                {}
+func (*EUnary) isExpr()                {}
+func (*EBinary) isExpr()               {}
+func (*EBoolean) isExpr()              {}
+func (*ESuper) isExpr()                {}
+func (*ENull) isExpr()                 {}
+func (*EUndefined) isExpr()            {}
+func (*EThis) isExpr()                 {}
+func (*ENew) isExpr()                  {}
+func (*ENewTarget) isExpr()            {}
+func (*EImportMeta) isExpr()           {}
+func (*ECall) isExpr()                 {}
+func (*EDot) isExpr()                  {}
+func (*EIndex) isExpr()                {}
+func (*EArrow) isExpr()                {}
+func (*EFunction) isExpr()             {}
+func (*EClass) isExpr()                {}
+func (*EIdentifier) isExpr()           {}
+func (*EImportIdentifier) isExpr()     {}
+func (*EPrivateIdentifier) isExpr()    {}
+func (*EJSXElement) isExpr()           {}
+func (*EMissing) isExpr()              {}
+func (*ENumber) isExpr()               {}
+func (*EBigInt) isExpr()               {}
+func (*EObject) isExpr()               {}
+func (*ESpread) isExpr()               {}
+func (*EString) isExpr()               {}
+func (*ETemplate) isExpr()             {}
+func (*ERegExp) isExpr()               {}
+func (*EAwait) isExpr()                {}
+func (*EYield) isExpr()                {}
+func (*EIf) isExpr()                   {}
+func (*ERequireString) isExpr()        {}
+func (*ERequireResolveString) isExpr() {}
+func (*EImportString) isExpr()         {}
+func (*EImportCall) isExpr()           {}
+
 type EArray struct {
 	Items            []Expr
 	CommaAfterSpread logger.Loc
@@ -661,43 +707,6 @@ type EImportCall struct {
 	LeadingInteriorComments []Comment
 }
 
-func (*EArray) isExpr()                {}
-func (*EUnary) isExpr()                {}
-func (*EBinary) isExpr()               {}
-func (*EBoolean) isExpr()              {}
-func (*ESuper) isExpr()                {}
-func (*ENull) isExpr()                 {}
-func (*EUndefined) isExpr()            {}
-func (*EThis) isExpr()                 {}
-func (*ENew) isExpr()                  {}
-func (*ENewTarget) isExpr()            {}
-func (*EImportMeta) isExpr()           {}
-func (*ECall) isExpr()                 {}
-func (*EDot) isExpr()                  {}
-func (*EIndex) isExpr()                {}
-func (*EArrow) isExpr()                {}
-func (*EFunction) isExpr()             {}
-func (*EClass) isExpr()                {}
-func (*EIdentifier) isExpr()           {}
-func (*EImportIdentifier) isExpr()     {}
-func (*EPrivateIdentifier) isExpr()    {}
-func (*EJSXElement) isExpr()           {}
-func (*EMissing) isExpr()              {}
-func (*ENumber) isExpr()               {}
-func (*EBigInt) isExpr()               {}
-func (*EObject) isExpr()               {}
-func (*ESpread) isExpr()               {}
-func (*EString) isExpr()               {}
-func (*ETemplate) isExpr()             {}
-func (*ERegExp) isExpr()               {}
-func (*EAwait) isExpr()                {}
-func (*EYield) isExpr()                {}
-func (*EIf) isExpr()                   {}
-func (*ERequireString) isExpr()        {}
-func (*ERequireResolveString) isExpr() {}
-func (*EImportString) isExpr()         {}
-func (*EImportCall) isExpr()           {}
-
 func IsOptionalChain(value Expr) bool {
 	switch e := value.Data.(type) {
 	case *EDot:
@@ -843,15 +852,19 @@ func IsNumericValue(a Expr) bool {
 		case BinOpAdd:
 			return IsNumericValue(e.Left) && IsNumericValue(e.Right)
 
-		case BinOpSub, BinOpMul, BinOpDiv, BinOpRem,
-			BinOpBitwiseAnd, BinOpBitwiseOr, BinOpBitwiseXor,
-			BinOpShl, BinOpShr, BinOpUShr:
+		case
+			BinOpSub, BinOpSubAssign,
+			BinOpMul, BinOpMulAssign,
+			BinOpDiv, BinOpDivAssign,
+			BinOpRem, BinOpRemAssign,
+			BinOpBitwiseAnd, BinOpBitwiseAndAssign,
+			BinOpBitwiseOr, BinOpBitwiseOrAssign,
+			BinOpBitwiseXor, BinOpBitwiseXorAssign,
+			BinOpShl, BinOpShlAssign,
+			BinOpShr, BinOpShrAssign,
+			BinOpUShr, BinOpUShrAssign:
 			return true
 
-		case BinOpSubAssign, BinOpMulAssign, BinOpDivAssign, BinOpRemAssign,
-			BinOpBitwiseAndAssign, BinOpBitwiseOrAssign, BinOpBitwiseXorAssign,
-			BinOpShlAssign, BinOpShrAssign, BinOpUShrAssign:
-
 		case BinOpAssign, BinOpComma:
 			return IsNumericValue(e.Right)
 		}
@@ -941,6 +954,40 @@ type Stmt struct {
 // Go's type system.
 type S interface{ isStmt() }
 
+func (*SBlock) isStmt()         {}
+func (*SComment) isStmt()       {}
+func (*SDebugger) isStmt()      {}
+func (*SDirective) isStmt()     {}
+func (*SEmpty) isStmt()         {}
+func (*STypeScript) isStmt()    {}
+func (*SExportClause) isStmt()  {}
+func (*SExportFrom) isStmt()    {}
+func (*SExportDefault) isStmt() {}
+func (*SExportStar) isStmt()    {}
+func (*SExportEquals) isStmt()  {}
+func (*SLazyExport) isStmt()    {}
+func (*SExpr) isStmt()          {}
+func (*SEnum) isStmt()          {}
+func (*SNamespace) isStmt()     {}
+func (*SFunction) isStmt()      {}
+func (*SClass) isStmt()         {}
+func (*SLabel) isStmt()         {}
+func (*SIf) isStmt()            {}
+func (*SFor) isStmt()           {}
+func (*SForIn) isStmt()         {}
+func (*SForOf) isStmt()         {}
+func (*SDoWhile) isStmt()       {}
+func (*SWhile) isStmt()         {}
+func (*SWith) isStmt()          {}
+func (*STry) isStmt()           {}
+func (*SSwitch) isStmt()        {}
+func (*SImport) isStmt()        {}
+func (*SReturn) isStmt()        {}
+func (*SThrow) isStmt()         {}
+func (*SLocal) isStmt()         {}
+func (*SBreak) isStmt()         {}
+func (*SContinue) isStmt()      {}
+
 type SBlock struct {
 	Stmts []Stmt
 }
@@ -1181,40 +1228,6 @@ type SContinue struct {
 	Label *LocRef
 }
 
-func (*SBlock) isStmt()         {}
-func (*SComment) isStmt()       {}
-func (*SDebugger) isStmt()      {}
-func (*SDirective) isStmt()     {}
-func (*SEmpty) isStmt()         {}
-func (*STypeScript) isStmt()    {}
-func (*SExportClause) isStmt()  {}
-func (*SExportFrom) isStmt()    {}
-func (*SExportDefault) isStmt() {}
-func (*SExportStar) isStmt()    {}
-func (*SExportEquals) isStmt()  {}
-func (*SLazyExport) isStmt()    {}
-func (*SExpr) isStmt()          {}
-func (*SEnum) isStmt()          {}
-func (*SNamespace) isStmt()     {}
-func (*SFunction) isStmt()      {}
-func (*SClass) isStmt()         {}
-func (*SLabel) isStmt()         {}
-func (*SIf) isStmt()            {}
-func (*SFor) isStmt()           {}
-func (*SForIn) isStmt()         {}
-func (*SForOf) isStmt()         {}
-func (*SDoWhile) isStmt()       {}
-func (*SWhile) isStmt()         {}
-func (*SWith) isStmt()          {}
-func (*STry) isStmt()           {}
-func (*SSwitch) isStmt()        {}
-func (*SImport) isStmt()        {}
-func (*SReturn) isStmt()        {}
-func (*SThrow) isStmt()         {}
-func (*SLocal) isStmt()         {}
-func (*SBreak) isStmt()         {}
-func (*SContinue) isStmt()      {}
-
 func IsSuperCall(stmt Stmt) bool {
 	if expr, ok := stmt.Data.(*SExpr); ok {
 		if call, ok := expr.Value.Data.(*ECall); ok {
@@ -1598,6 +1611,7 @@ const (
 	ScopeEntry // This is a module, TypeScript enum, or TypeScript namespace
 	ScopeFunctionArgs
 	ScopeFunctionBody
+	ScopeClassStaticInit
 )
 
 func (kind ScopeKind) StopsHoisting() bool {
