@@ -20,6 +20,7 @@ import (
 	"unicode"
 	"unicode/utf8"
 
+	"github.com/evanw/esbuild/internal/helpers"
 	"github.com/evanw/esbuild/internal/js_ast"
 	"github.com/evanw/esbuild/internal/logger"
 )
@@ -577,17 +578,17 @@ func IsIdentifier(text string) bool {
 	return true
 }
 
-func IsIdentifierES5(text string) bool {
+func IsIdentifierES5AndESNext(text string) bool {
 	if len(text) == 0 {
 		return false
 	}
 	for i, codePoint := range text {
 		if i == 0 {
-			if !IsIdentifierStartES5(codePoint) {
+			if !IsIdentifierStartES5AndESNext(codePoint) {
 				return false
 			}
 		} else {
-			if !IsIdentifierContinueES5(codePoint) {
+			if !IsIdentifierContinueES5AndESNext(codePoint) {
 				return false
 			}
 		}
@@ -652,8 +653,8 @@ func IsIdentifierUTF16(text []uint16) bool {
 	return true
 }
 
-// This does "IsIdentifierES5(UTF16ToString(text))" without any allocations
-func IsIdentifierES5UTF16(text []uint16) bool {
+// This does "IsIdentifierES5AndESNext(UTF16ToString(text))" without any allocations
+func IsIdentifierES5AndESNextUTF16(text []uint16) bool {
 	n := len(text)
 	if n == 0 {
 		return false
@@ -668,11 +669,11 @@ func IsIdentifierES5UTF16(text []uint16) bool {
 			}
 		}
 		if isStart {
-			if !IsIdentifierStartES5(r1) {
+			if !IsIdentifierStartES5AndESNext(r1) {
 				return false
 			}
 		} else {
-			if !IsIdentifierContinueES5(r1) {
+			if !IsIdentifierContinueES5AndESNext(r1) {
 				return false
 			}
 		}
@@ -695,7 +696,7 @@ func IsIdentifierStart(codePoint rune) bool {
 		return false
 	}
 
-	return unicode.Is(idStart, codePoint)
+	return unicode.Is(idStartES5OrESNext, codePoint)
 }
 
 func IsIdentifierContinue(codePoint rune) bool {
@@ -718,10 +719,10 @@ func IsIdentifierContinue(codePoint rune) bool {
 		return true
 	}
 
-	return unicode.Is(idContinue, codePoint)
+	return unicode.Is(idContinueES5OrESNext, codePoint)
 }
 
-func IsIdentifierStartES5(codePoint rune) bool {
+func IsIdentifierStartES5AndESNext(codePoint rune) bool {
 	switch codePoint {
 	case '_', '$',
 		'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
@@ -736,10 +737,10 @@ func IsIdentifierStartES5(codePoint rune) bool {
 		return false
 	}
 
-	return unicode.Is(idStartES5, codePoint)
+	return unicode.Is(idStartES5AndESNext, codePoint)
 }
 
-func IsIdentifierContinueES5(codePoint rune) bool {
+func IsIdentifierContinueES5AndESNext(codePoint rune) bool {
 	switch codePoint {
 	case '_', '$', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
 		'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
@@ -759,7 +760,7 @@ func IsIdentifierContinueES5(codePoint rune) bool {
 		return true
 	}
 
-	return unicode.Is(idContinueES5, codePoint)
+	return unicode.Is(idContinueES5AndESNext, codePoint)
 }
 
 // See the "White Space Code Points" table in the ECMAScript standard
@@ -2729,7 +2730,7 @@ func (lexer *Lexer) scanCommentText() {
 
 	if hasLegalAnnotation || lexer.PreserveAllCommentsBefore {
 		if isMultiLineComment {
-			text = removeMultiLineCommentIndent(lexer.source.Contents[:lexer.start], text)
+			text = helpers.RemoveMultiLineCommentIndent(lexer.source.Contents[:lexer.start], text)
 		}
 
 		lexer.CommentsToPreserveBefore = append(lexer.CommentsToPreserveBefore, js_ast.Comment{
@@ -2739,68 +2740,6 @@ func (lexer *Lexer) scanCommentText() {
 	}
 }
 
-func removeMultiLineCommentIndent(prefix string, text string) string {
-	// Figure out the initial indent
-	indent := 0
-seekBackwardToNewline:
-	for len(prefix) > 0 {
-		c, size := utf8.DecodeLastRuneInString(prefix)
-		switch c {
-		case '\r', '\n', '\u2028', '\u2029':
-			break seekBackwardToNewline
-		}
-		prefix = prefix[:len(prefix)-size]
-		indent++
-	}
-
-	// Split the comment into lines
-	var lines []string
-	start := 0
-	for i, c := range text {
-		switch c {
-		case '\r', '\n':
-			// Don't double-append for Windows style "\r\n" newlines
-			if start <= i {
-				lines = append(lines, text[start:i])
-			}
-
-			start = i + 1
-
-			// Ignore the second part of Windows style "\r\n" newlines
-			if c == '\r' && start < len(text) && text[start] == '\n' {
-				start++
-			}
-
-		case '\u2028', '\u2029':
-			lines = append(lines, text[start:i])
-			start = i + 3
-		}
-	}
-	lines = append(lines, text[start:])
-
-	// Find the minimum indent over all lines after the first line
-	for _, line := range lines[1:] {
-		lineIndent := 0
-		for _, c := range line {
-			if !IsWhitespace(c) {
-				break
-			}
-			lineIndent++
-		}
-		if indent > lineIndent {
-			indent = lineIndent
-		}
-	}
-
-	// Trim the indent off of all lines after the first line
-	for i, line := range lines {
-		if i > 0 {
-			lines[i] = line[indent:]
-		}
-	}
-	return strings.Join(lines, "\n")
-}
-
 func ContainsNonBMPCodePoint(text string) bool {
 	for _, c := range text {
 		if c > 0xFFFF {
