@@ -46,7 +46,7 @@ func GetTerminalInfo(file *os.File) TerminalInfo {
 		IsTTY:           isTTY != 0,
 		Width:           int(info.dwSizeX) - 1,
 		Height:          int(info.dwSizeY) - 1,
-		UseColorEscapes: true,
+		UseColorEscapes: !hasNoColorEnvironmentVariable(),
 	}
 }
 
