@@ -144,10 +144,19 @@ func parseOptionsImpl(
 			}
 			name := arg[len("--tree-shaking="):]
 			switch name {
-			case "ignore-annotations":
-				*value = api.TreeShakingIgnoreAnnotations
+			case "false":
+				*value = api.TreeShakingFalse
+			case "true":
+				*value = api.TreeShakingTrue
 			default:
-				return fmt.Errorf("Invalid tree shaking value: %q (valid: ignore-annotations)", name), nil
+				return fmt.Errorf("Invalid tree shaking value: %q (valid: false, true)", name), nil
+			}
+
+		case arg == "--ignore-annotations":
+			if buildOpts != nil {
+				buildOpts.IgnoreAnnotations = true
+			} else {
+				transformOpts.IgnoreAnnotations = true
 			}
 
 		case arg == "--keep-names":
@@ -558,7 +567,7 @@ func parseTargets(targets []string) (target api.Target, engines []api.Engine, er
 
 outer:
 	for _, value := range targets {
-		if valid, ok := validTargets[value]; ok {
+		if valid, ok := validTargets[strings.ToLower(value)]; ok {
 			target = valid
 			continue
 		}
@@ -623,6 +632,8 @@ func parseOptionsForRun(osArgs []string) (*api.BuildOptions, *string, *api.Trans
 }
 
 func runImpl(osArgs []string) int {
+	analyze := false
+	analyzeVerbose := false
 	end := 0
 
 	for _, arg := range osArgs {
@@ -635,6 +646,18 @@ func runImpl(osArgs []string) int {
 			return 0
 		}
 
+		// Special-case analyze just for our CLI
+		if arg == "--analyze" {
+			analyze = true
+			analyzeVerbose = false
+			continue
+		}
+		if arg == "--analyze=verbose" {
+			analyze = true
+			analyzeVerbose = true
+			continue
+		}
+
 		osArgs[end] = arg
 		end++
 	}
@@ -658,12 +681,6 @@ func runImpl(osArgs []string) int {
 				buildOptions.NodePaths = strings.Split(value, separator)
 				break
 			}
-
-			// Read "NO_COLOR" from the environment. This is a convention that some
-			// software follows. See https://no-color.org/ for more information.
-			if buildOptions.Color == api.ColorIfTerminal && strings.HasPrefix(key, "NO_COLOR=") {
-				buildOptions.Color = api.ColorNever
-			}
 		}
 
 		// Read from stdin when there are no entry points
@@ -745,9 +762,25 @@ func runImpl(osArgs []string) int {
 			}
 		}
 
+		// Always generate a metafile if we're analyzing, even if it won't be written out
+		if analyze {
+			buildOptions.Metafile = true
+		}
+
 		// Run the build
 		result := api.Build(*buildOptions)
 
+		// Print the analysis after the build
+		if analyze {
+			logger.PrintTextWithColor(os.Stderr, logger.OutputOptionsForArgs(osArgs).Color, func(colors logger.Colors) string {
+				return api.AnalyzeMetafile(result.Metafile, api.AnalyzeMetafileOptions{
+					Color:   colors != logger.Colors{},
+					Verbose: analyzeVerbose,
+				})
+			})
+			os.Stderr.WriteString("\n")
+		}
+
 		// Write the metafile to the file system
 		if writeMetafile != nil {
 			writeMetafile(result.Metafile)
