@@ -709,6 +709,80 @@ let pluginTests = {
     assert.strictEqual(result.default, 123)
   },
 
+  async resolveWithSideEffectsFalse({ esbuild, testDir }) {
+    const input = path.join(testDir, 'in.js')
+
+    await writeFileAsync(input, `
+      import './re-export-unused'
+      import {a, b, c} from './re-export-used'
+      import './import-unused'
+      use([a, b, c])
+    `)
+    await writeFileAsync(path.join(testDir, 're-export-unused.js'), `
+      export {default as a} from 'plugin:unused-false'
+      export {default as b} from 'plugin:unused-true'
+      export {default as c} from 'plugin:unused-none'
+    `)
+    await writeFileAsync(path.join(testDir, 're-export-used.js'), `
+      export {default as a} from 'plugin:used-false'
+      export {default as b} from 'plugin:used-true'
+      export {default as c} from 'plugin:used-none'
+    `)
+    await writeFileAsync(path.join(testDir, 'import-unused.js'), `
+      import 'plugin:ignored-false'
+      import 'plugin:ignored-true'
+      import 'plugin:ignored-none'
+    `)
+
+    const result = await esbuild.build({
+      entryPoints: [input],
+      bundle: true,
+      write: false,
+      format: 'cjs',
+      logLevel: 'error',
+      plugins: [{
+        name: 'name',
+        setup(build) {
+          build.onResolve({ filter: /^plugin:/ }, args => {
+            return {
+              path: args.path,
+              namespace: 'ns',
+              sideEffects:
+                args.path.endsWith('-true') ? true :
+                  args.path.endsWith('-false') ? false :
+                    undefined,
+            };
+          });
+          build.onLoad({ filter: /^plugin:/ }, args => {
+            return { contents: `export default use(${JSON.stringify(args.path)})` };
+          });
+        },
+      }],
+    })
+
+    // Validate that the unused "sideEffects: false" files were omitted
+    const used = [];
+    new Function('use', result.outputFiles[0].text)(x => used.push(x));
+    assert.deepStrictEqual(used, [
+      'plugin:unused-true',
+      'plugin:unused-none',
+
+      'plugin:used-false',
+      'plugin:used-true',
+      'plugin:used-none',
+
+      'plugin:ignored-true',
+      'plugin:ignored-none',
+
+      [3, 4, 5],
+    ])
+
+    // Check that the warning for "sideEffect: false" imports mentions the plugin
+    assert.strictEqual(result.warnings.length, 1)
+    assert.strictEqual(result.warnings[0].text,
+      'Ignoring this import because "ns:plugin:ignored-false" was marked as having no side effects by plugin "name"')
+  },
+
   async noResolveDirInFileModule({ esbuild, testDir }) {
     const input = path.join(testDir, 'in.js')
     const output = path.join(testDir, 'out.js')
@@ -1772,6 +1846,7 @@ let pluginTests = {
         bundle: true,
         write: false,
         logLevel: 'silent',
+        platform: 'node',
         plugins: [{
           name: 'plugin',
           setup(build) {
@@ -1929,6 +2004,39 @@ let pluginTests = {
       }],
     })
   },
+
+  async fileLoaderCustomNamespaceIssue1404({ esbuild, testDir }) {
+    const input = path.join(testDir, 'in.data')
+    const outdir = path.join(testDir, 'out')
+    await writeFileAsync(input, `some data`)
+    await esbuild.build({
+      entryPoints: [path.basename(input)],
+      absWorkingDir: testDir,
+      logLevel: 'silent',
+      outdir,
+      assetNames: '[name]',
+      plugins: [{
+        name: 'plugin',
+        setup(build) {
+          build.onResolve({ filter: /\.data$/ }, args => {
+            return {
+              path: args.path,
+              namespace: 'ns',
+            }
+          })
+          build.onLoad({ filter: /.*/, namespace: 'ns' }, async (args) => {
+            const data = await readFileAsync(path.join(testDir, args.path), 'utf8')
+            return {
+              contents: data.split('').reverse().join(''),
+              loader: 'file',
+            }
+          })
+        },
+      }],
+    })
+    assert.strictEqual(await readFileAsync(input, 'utf8'), `some data`)
+    assert.strictEqual(require(path.join(outdir, 'in.js')), `./in.data`)
+  },
 }
 
 // These tests have to run synchronously
@@ -2328,6 +2436,7 @@ let syncTests = {
         onRebuild: (...args) => onRebuild(args),
       },
       bundle: true,
+      metafile: true,
       plugins: [
         {
           name: 'some-plugin',
@@ -2335,7 +2444,8 @@ let syncTests = {
             build.onStart(() => {
               onStartCalls++
             })
-            build.onEnd(() => {
+            build.onEnd(result => {
+              assert.notStrictEqual(result.metafile, void 0)
               onEndCalls++
             })
 
@@ -2404,7 +2514,7 @@ async function main() {
   // Time out these tests after 5 minutes. This exists to help debug test hangs in CI.
   let minutes = 5
   let timeout = setTimeout(() => {
-    console.error(`❌ js api tests timed out after ${minutes} minutes, exiting...`)
+    console.error(`❌ plugin tests timed out after ${minutes} minutes, exiting...`)
     process.exit(1)
   }, minutes * 60 * 1000)
 
